@@ -15,18 +15,19 @@
  */
 package org.joda.time;
 
+import java.io.File;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.ObjectStreamException;
 import java.io.Serializable;
-import java.lang.ref.Reference;
-import java.lang.ref.SoftReference;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import java.util.TimeZone;
+import java.util.concurrent.atomic.AtomicReference;
 
 import org.joda.convert.FromString;
 import org.joda.convert.ToString;
@@ -78,6 +79,19 @@ import org.joda.time.tz.ZoneInfoProvider;
  * However, the factory that accepts a TimeZone will attempt to convert from
  * the old short id to a suitable long id.
  * <p>
+ * There are four approaches to loading time-zone data, which are tried in this order:
+ * <ol>
+ * <li>load the specific {@link Provider} specified by the system property
+ *   {@code org.joda.time.DateTimeZone.Provider}.
+ * <li>load {@link ZoneInfoProvider} using the data in the filing system folder
+ *   pointed to by system property {@code org.joda.time.DateTimeZone.Folder}.
+ * <li>load {@link ZoneInfoProvider} using the data in the classpath location
+ *   {@code org/joda/time/tz/data}.
+ * <li>load {@link UTCProvider}
+ * </ol>
+ * <p>
+ * Unless you override the standard behaviour, the default if the third approach.
+ * <p>
  * DateTimeZone is thread-safe and immutable, and all subclasses must be as
  * well.
  * 
@@ -91,31 +105,28 @@ public abstract class DateTimeZone implements Serializable {
     private static final long serialVersionUID = 5546345482340108586L;
 
     /** The time zone for Universal Coordinated Time */
-    public static final DateTimeZone UTC = new FixedDateTimeZone("UTC", "UTC", 0, 0);
+    public static final DateTimeZone UTC = UTCDateTimeZone.INSTANCE;
     /** Maximum offset. */
     private static final int MAX_MILLIS = (86400 * 1000) - 1;
 
-    /** The instance that is providing time zones. */
-    private static Provider cProvider;
-    /** The instance that is providing time zone names. */
-    private static NameProvider cNameProvider;
-    /** The set of ID strings. */
-    private static Set<String> cAvailableIDs;
-    /** The default time zone. */
-    private static volatile DateTimeZone cDefault;
-    /** A formatter for printing and parsing zones. */
-    private static DateTimeFormatter cOffsetFormatter;
-
-    /** Cache that maps fixed offset strings to softly referenced DateTimeZones */
-    private static Map<String, SoftReference<DateTimeZone>> iFixedOffsetCache;
-
-    /** Cache of old zone IDs to new zone IDs */
-    private static Map<String, String> cZoneIdConversion;
-
-    static {
-        setProvider0(null);
-        setNameProvider0(null);
-    }
+    /**
+     * The instance that is providing time zones.
+     * This is lazily initialized to reduce risks of race conditions at startup.
+     */
+    private static final AtomicReference<Provider> cProvider =
+                    new AtomicReference<Provider>();
+    /**
+     * The instance that is providing time zone names.
+     * This is lazily initialized to reduce risks of race conditions at startup.
+     */
+    private static final AtomicReference<NameProvider> cNameProvider =
+                    new AtomicReference<NameProvider>();
+    /**
+     * The default time zone.
+     * This is lazily initialized to reduce risks of race conditions at startup.
+     */
+    private static final AtomicReference<DateTimeZone> cDefault =
+                    new AtomicReference<DateTimeZone>();
 
     //-----------------------------------------------------------------------
     /**
@@ -131,32 +142,28 @@ public abstract class DateTimeZone implements Serializable {
      * @return the default datetime zone object
      */
     public static DateTimeZone getDefault() {
-        DateTimeZone zone = cDefault;
+        DateTimeZone zone = cDefault.get();
         if (zone == null) {
-            synchronized(DateTimeZone.class) {
-                zone = cDefault;
-                if (zone == null) {
-                    DateTimeZone temp = null;
-                    try {
-                        try {
-                            String id = System.getProperty("user.timezone");
-                            if (id != null) {  // null check avoids stack overflow
-                                temp = forID(id);
-                            }
-                        } catch (RuntimeException ex) {
-                            // ignored
-                        }
-                        if (temp == null) {
-                            temp = forTimeZone(TimeZone.getDefault());
-                        }
-                    } catch (IllegalArgumentException ex) {
-                        // ignored
+            try {
+                try {
+                    String id = System.getProperty("user.timezone");
+                    if (id != null) {  // null check avoids stack overflow
+                        zone = forID(id);
                     }
-                    if (temp == null) {
-                        temp = UTC;
-                    }
-                    cDefault = zone = temp;
+                } catch (RuntimeException ex) {
+                    // ignored
                 }
+                if (zone == null) {
+                    zone = forTimeZone(TimeZone.getDefault());
+                }
+            } catch (IllegalArgumentException ex) {
+                // ignored
+            }
+            if (zone == null) {
+                zone = UTC;
+            }
+            if (!cDefault.compareAndSet(null, zone)) {
+                zone = cDefault.get();
             }
         }
         return zone;
@@ -179,9 +186,7 @@ public abstract class DateTimeZone implements Serializable {
         if (zone == null) {
             throw new IllegalArgumentException("The datetime zone must not be null");
         }
-        synchronized(DateTimeZone.class) {
-            cDefault = zone;
-        }
+        cDefault.set(zone);
     }
 
     //-----------------------------------------------------------------------
@@ -208,7 +213,7 @@ public abstract class DateTimeZone implements Serializable {
         if (id.equals("UTC")) {
             return DateTimeZone.UTC;
         }
-        DateTimeZone zone = cProvider.getZone(id);
+        DateTimeZone zone = getProvider().getZone(id);
         if (zone != null) {
             return zone;
         }
@@ -341,11 +346,12 @@ public abstract class DateTimeZone implements Serializable {
         // Convert from old alias before consulting provider since they may differ.
         DateTimeZone dtz = null;
         String convId = getConvertedId(id);
+        Provider provider = getProvider();
         if (convId != null) {
-            dtz = cProvider.getZone(convId);
+            dtz = provider.getZone(convId);
         }
         if (dtz == null) {
-            dtz = cProvider.getZone(id);
+            dtz = provider.getZone(id);
         }
         if (dtz != null) {
             return dtz;
@@ -356,6 +362,12 @@ public abstract class DateTimeZone implements Serializable {
             convId = id;
             if (convId.startsWith("GMT+") || convId.startsWith("GMT-")) {
                 convId = convId.substring(3);
+                if (convId.length() > 2) {
+                    char firstDigit = convId.charAt(1);
+                    if (firstDigit > '9' && Character.isDigit(firstDigit)) {
+                        convId = convertToAsciiNumber(convId);
+                    }
+                }
                 int offset = parseOffset(convId);
                 if (offset == 0L) {
                     return DateTimeZone.UTC;
@@ -368,6 +380,18 @@ public abstract class DateTimeZone implements Serializable {
         throw new IllegalArgumentException("The datetime zone id '" + id + "' is not recognised");
     }
 
+    private static String convertToAsciiNumber(String convId) {
+        StringBuilder buf = new StringBuilder(convId);
+        for (int i = 0; i < buf.length(); i++) {
+            char ch = buf.charAt(i);
+            int digit = Character.digit(ch, 10);
+            if (digit >= 0) {
+                buf.setCharAt(i, (char) ('0' + digit));
+            }
+        }
+        return buf.toString();
+    }
+
     //-----------------------------------------------------------------------
     /**
      * Gets the zone using a fixed offset amount.
@@ -376,24 +400,11 @@ public abstract class DateTimeZone implements Serializable {
      * @param offset  the offset in millis
      * @return the zone
      */
-    private static synchronized DateTimeZone fixedOffsetZone(String id, int offset) {
+    private static DateTimeZone fixedOffsetZone(String id, int offset) {
         if (offset == 0) {
             return DateTimeZone.UTC;
         }
-        if (iFixedOffsetCache == null) {
-            iFixedOffsetCache = new HashMap<String, SoftReference<DateTimeZone>>();
-        }
-        DateTimeZone zone;
-        Reference<DateTimeZone> ref = iFixedOffsetCache.get(id);
-        if (ref != null) {
-            zone = ref.get();
-            if (zone != null) {
-                return zone;
-            }
-        }
-        zone = new FixedDateTimeZone(id, null, offset, offset);
-        iFixedOffsetCache.put(id, new SoftReference<DateTimeZone>(zone));
-        return zone;
+        return new FixedDateTimeZone(id, null, offset, offset);
     }
 
     /**
@@ -402,7 +413,7 @@ public abstract class DateTimeZone implements Serializable {
      * @return an unmodifiable Set of String IDs
      */
     public static Set<String> getAvailableIDs() {
-        return cAvailableIDs;
+        return getProvider().getAvailableIDs();
     }
 
     //-----------------------------------------------------------------------
@@ -415,7 +426,14 @@ public abstract class DateTimeZone implements Serializable {
      * @return the provider
      */
     public static Provider getProvider() {
-        return cProvider;
+        Provider provider = cProvider.get();
+        if (provider == null) {
+            provider = getDefaultProvider();
+            if (!cProvider.compareAndSet(null, provider)) {
+                provider = cProvider.get();
+            }
+        }
+        return provider;
     }
 
     /**
@@ -433,23 +451,25 @@ public abstract class DateTimeZone implements Serializable {
         if (sm != null) {
             sm.checkPermission(new JodaTimePermission("DateTimeZone.setProvider"));
         }
-        setProvider0(provider);
+        if (provider == null) {
+            provider = getDefaultProvider();
+        } else {
+            validateProvider(provider);
+        }
+        cProvider.set(provider);
     }
 
     /**
      * Sets the zone provider factory without performing the security check.
      * 
      * @param provider  provider to use, or null for default
+     * @return the provider
      * @throws IllegalArgumentException if the provider is invalid
      */
-    private static void setProvider0(Provider provider) {
-        if (provider == null) {
-            provider = getDefaultProvider();
-        }
+    private static Provider validateProvider(Provider provider) {
         Set<String> ids = provider.getAvailableIDs();
         if (ids == null || ids.size() == 0) {
-            throw new IllegalArgumentException
-                ("The provider doesn't have any available ids");
+            throw new IllegalArgumentException("The provider doesn't have any available ids");
         }
         if (!ids.contains("UTC")) {
             throw new IllegalArgumentException("The provider doesn't support UTC");
@@ -457,27 +477,35 @@ public abstract class DateTimeZone implements Serializable {
         if (!UTC.equals(provider.getZone("UTC"))) {
             throw new IllegalArgumentException("Invalid UTC zone provided");
         }
-        cProvider = provider;
-        cAvailableIDs = ids;
+        return provider;
     }
 
     /**
      * Gets the default zone provider.
      * <p>
-     * Tries the system property <code>org.joda.time.DateTimeZone.Provider</code>.
-     * Then tries a <code>ZoneInfoProvider</code> using the data in <code>org/joda/time/tz/data</code>.
-     * Then uses <code>UTCProvider</code>.
+     * This tries four approaches to loading data:
+     * <ol>
+     * <li>loads the provider identifier by the system property
+     *   <code>org.joda.time.DateTimeZone.Provider</code>.
+     * <li>load <code>ZoneInfoProvider</code> using the data in the filing system folder
+     *   pointed to by system property <code>org.joda.time.DateTimeZone.Folder</code>.
+     * <li>loads <code>ZoneInfoProvider</code> using the data in the classpath location
+     *   <code>org/joda/time/tz/data</code>.
+     * <li>loads <code>UTCProvider</code>.
+     * </ol>
+     * <p>
+     * Unless you override the standard behaviour, the default if the third approach.
      * 
      * @return the default name provider
      */
     private static Provider getDefaultProvider() {
-        Provider provider = null;
-
+        // approach 1
         try {
             String providerClass = System.getProperty("org.joda.time.DateTimeZone.Provider");
             if (providerClass != null) {
                 try {
-                    provider = (Provider) Class.forName(providerClass).newInstance();
+                    Provider provider = (Provider) Class.forName(providerClass).newInstance();
+                    return validateProvider(provider);
                 } catch (Exception ex) {
                     throw new RuntimeException(ex);
                 }
@@ -485,20 +513,29 @@ public abstract class DateTimeZone implements Serializable {
         } catch (SecurityException ex) {
             // ignored
         }
-
-        if (provider == null) {
-            try {
-                provider = new ZoneInfoProvider("org/joda/time/tz/data");
-            } catch (Exception ex) {
-                ex.printStackTrace();
+        // approach 2
+        try {
+            String dataFolder = System.getProperty("org.joda.time.DateTimeZone.Folder");
+            if (dataFolder != null) {
+                try {
+                    Provider provider = new ZoneInfoProvider(new File(dataFolder));
+                    return validateProvider(provider);
+                } catch (Exception ex) {
+                    throw new RuntimeException(ex);
+                }
             }
+        } catch (SecurityException ex) {
+            // ignored
         }
-
-        if (provider == null) {
-            provider = new UTCProvider();
+        // approach 3
+        try {
+            Provider provider = new ZoneInfoProvider("org/joda/time/tz/data");
+            return validateProvider(provider);
+        } catch (Exception ex) {
+            ex.printStackTrace();
         }
-
-        return provider;
+        // approach 4
+        return new UTCProvider();
     }
 
     //-----------------------------------------------------------------------
@@ -511,7 +548,14 @@ public abstract class DateTimeZone implements Serializable {
      * @return the provider
      */
     public static NameProvider getNameProvider() {
-        return cNameProvider;
+        NameProvider nameProvider = cNameProvider.get();
+        if (nameProvider == null) {
+            nameProvider = getDefaultNameProvider();
+            if (!cNameProvider.compareAndSet(null, nameProvider)) {
+                nameProvider = cNameProvider.get();
+            }
+        }
+        return nameProvider;
     }
 
     /**
@@ -529,20 +573,10 @@ public abstract class DateTimeZone implements Serializable {
         if (sm != null) {
             sm.checkPermission(new JodaTimePermission("DateTimeZone.setNameProvider"));
         }
-        setNameProvider0(nameProvider);
-    }
-
-    /**
-     * Sets the name provider factory without performing the security check.
-     * 
-     * @param nameProvider  provider to use, or null for default
-     * @throws IllegalArgumentException if the provider is invalid
-     */
-    private static void setNameProvider0(NameProvider nameProvider) {
         if (nameProvider == null) {
             nameProvider = getDefaultNameProvider();
         }
-        cNameProvider = nameProvider;
+        cNameProvider.set(nameProvider);
     }
 
     /**
@@ -582,68 +616,18 @@ public abstract class DateTimeZone implements Serializable {
      * @param id  the old style id
      * @return the new style id, null if not found
      */
-    private static synchronized String getConvertedId(String id) {
-        Map<String, String> map = cZoneIdConversion;
-        if (map == null) {
-            // Backwards compatibility with TimeZone.
-            map = new HashMap<String, String>();
-            map.put("GMT", "UTC");
-            map.put("WET", "WET");
-            map.put("CET", "CET");
-            map.put("MET", "CET");
-            map.put("ECT", "CET");
-            map.put("EET", "EET");
-            map.put("MIT", "Pacific/Apia");
-            map.put("HST", "Pacific/Honolulu");  // JDK 1.1 compatible
-            map.put("AST", "America/Anchorage");
-            map.put("PST", "America/Los_Angeles");
-            map.put("MST", "America/Denver");  // JDK 1.1 compatible
-            map.put("PNT", "America/Phoenix");
-            map.put("CST", "America/Chicago");
-            map.put("EST", "America/New_York");  // JDK 1.1 compatible
-            map.put("IET", "America/Indiana/Indianapolis");
-            map.put("PRT", "America/Puerto_Rico");
-            map.put("CNT", "America/St_Johns");
-            map.put("AGT", "America/Argentina/Buenos_Aires");
-            map.put("BET", "America/Sao_Paulo");
-            map.put("ART", "Africa/Cairo");
-            map.put("CAT", "Africa/Harare");
-            map.put("EAT", "Africa/Addis_Ababa");
-            map.put("NET", "Asia/Yerevan");
-            map.put("PLT", "Asia/Karachi");
-            map.put("IST", "Asia/Kolkata");
-            map.put("BST", "Asia/Dhaka");
-            map.put("VST", "Asia/Ho_Chi_Minh");
-            map.put("CTT", "Asia/Shanghai");
-            map.put("JST", "Asia/Tokyo");
-            map.put("ACT", "Australia/Darwin");
-            map.put("AET", "Australia/Sydney");
-            map.put("SST", "Pacific/Guadalcanal");
-            map.put("NST", "Pacific/Auckland");
-            cZoneIdConversion = map;
-        }
-        return map.get(id);
+    private static String getConvertedId(String id) {
+        return LazyInit.CONVERSION_MAP.get(id);
     }
 
+    /**
+     * Parses an offset from the string.
+     * 
+     * @param str  the string
+     * @return the offset millis
+     */
     private static int parseOffset(String str) {
-        // Can't use a real chronology if called during class
-        // initialization. Offset parser doesn't need it anyhow.
-        Chronology chrono = new BaseChronology() {
-            private static final long serialVersionUID = -3128740902654445468L;
-            public DateTimeZone getZone() {
-                return null;
-            }
-            public Chronology withUTC() {
-                return this;
-            }
-            public Chronology withZone(DateTimeZone zone) {
-                return this;
-            }
-            public String toString() {
-                return getClass().getName();
-            }
-        };
-        return -(int) offsetFormatter().withChronology(chrono).parseMillis(str);
+        return -(int) LazyInit.OFFSET_FORMATTER.parseMillis(str);
     }
 
     /**
@@ -689,20 +673,6 @@ public abstract class DateTimeZone implements Serializable {
         return buf.toString();
     }
 
-    /**
-     * Gets a printer/parser for managing the offset id formatting.
-     * 
-     * @return the formatter
-     */
-    private static synchronized DateTimeFormatter offsetFormatter() {
-        if (cOffsetFormatter == null) {
-            cOffsetFormatter = new DateTimeFormatterBuilder()
-                .appendTimeZoneOffset(null, true, 2, 4)
-                .toFormatter();
-        }
-        return cOffsetFormatter;
-    }
-
     // Instance fields and methods
     //--------------------------------------------------------------------
 
@@ -776,7 +746,13 @@ public abstract class DateTimeZone implements Serializable {
         if (nameKey == null) {
             return iID;
         }
-        String name = cNameProvider.getShortName(locale, iID, nameKey);
+        String name;
+        NameProvider np = getNameProvider();
+        if (np instanceof DefaultNameProvider) {
+            name = ((DefaultNameProvider) np).getShortName(locale, iID, nameKey, isStandardOffset(instant));
+        } else {
+            name = np.getShortName(locale, iID, nameKey);
+        }
         if (name != null) {
             return name;
         }
@@ -816,7 +792,13 @@ public abstract class DateTimeZone implements Serializable {
         if (nameKey == null) {
             return iID;
         }
-        String name = cNameProvider.getName(locale, iID, nameKey);
+        String name;
+        NameProvider np = getNameProvider();
+        if (np instanceof DefaultNameProvider) {
+            name = ((DefaultNameProvider) np).getName(locale, iID, nameKey, isStandardOffset(instant));
+        } else {
+            name = np.getName(locale, iID, nameKey);
+        }
         if (name != null) {
             return name;
         }
@@ -1333,4 +1315,79 @@ public abstract class DateTimeZone implements Serializable {
         }
     }
 
+    //-------------------------------------------------------------------------
+    /**
+     * Lazy initialization to avoid a synchronization lock.
+     */
+    static final class LazyInit {
+
+        /** Cache of old zone IDs to new zone IDs */
+        static final Map<String, String> CONVERSION_MAP = buildMap();
+        /** Time zone offset formatter. */
+        static final DateTimeFormatter OFFSET_FORMATTER = buildFormatter();
+
+        private static DateTimeFormatter buildFormatter() {
+            // Can't use a real chronology if called during class
+            // initialization. Offset parser doesn't need it anyhow.
+            Chronology chrono = new BaseChronology() {
+                private static final long serialVersionUID = -3128740902654445468L;
+                public DateTimeZone getZone() {
+                    return null;
+                }
+                public Chronology withUTC() {
+                    return this;
+                }
+                public Chronology withZone(DateTimeZone zone) {
+                    return this;
+                }
+                public String toString() {
+                    return getClass().getName();
+                }
+            };
+            return new DateTimeFormatterBuilder()
+                .appendTimeZoneOffset(null, true, 2, 4)
+                .toFormatter()
+                .withChronology(chrono);
+        }
+
+        private static Map<String, String> buildMap() {
+            // Backwards compatibility with TimeZone.
+            Map<String, String> map = new HashMap<String, String>();
+            map.put("GMT", "UTC");
+            map.put("WET", "WET");
+            map.put("CET", "CET");
+            map.put("MET", "CET");
+            map.put("ECT", "CET");
+            map.put("EET", "EET");
+            map.put("MIT", "Pacific/Apia");
+            map.put("HST", "Pacific/Honolulu");  // JDK 1.1 compatible
+            map.put("AST", "America/Anchorage");
+            map.put("PST", "America/Los_Angeles");
+            map.put("MST", "America/Denver");  // JDK 1.1 compatible
+            map.put("PNT", "America/Phoenix");
+            map.put("CST", "America/Chicago");
+            map.put("EST", "America/New_York");  // JDK 1.1 compatible
+            map.put("IET", "America/Indiana/Indianapolis");
+            map.put("PRT", "America/Puerto_Rico");
+            map.put("CNT", "America/St_Johns");
+            map.put("AGT", "America/Argentina/Buenos_Aires");
+            map.put("BET", "America/Sao_Paulo");
+            map.put("ART", "Africa/Cairo");
+            map.put("CAT", "Africa/Harare");
+            map.put("EAT", "Africa/Addis_Ababa");
+            map.put("NET", "Asia/Yerevan");
+            map.put("PLT", "Asia/Karachi");
+            map.put("IST", "Asia/Kolkata");
+            map.put("BST", "Asia/Dhaka");
+            map.put("VST", "Asia/Ho_Chi_Minh");
+            map.put("CTT", "Asia/Shanghai");
+            map.put("JST", "Asia/Tokyo");
+            map.put("ACT", "Australia/Darwin");
+            map.put("AET", "Australia/Sydney");
+            map.put("SST", "Pacific/Guadalcanal");
+            map.put("NST", "Pacific/Auckland");
+            return Collections.unmodifiableMap(map);
+        }
+    }
+
 }
